@@ -36,6 +36,7 @@ from common import helpers
 from common import gajim
 from common import GnuPG
 from common import passwords
+from common import exceptions
 
 from connection_handlers import *
 USE_GPG = GnuPG.USE_GPG
@@ -78,8 +79,12 @@ class Connection(ConnectionHandlers):
 		self.privacy_rules_supported = False
 		# Do we continue connection when we get roster (send presence,get vcard...)
 		self.continue_connect_info = None
+		# To know the groupchat jid associated with a sranza ID. Useful to
+		# request vcard or os info... to a real JID but act as if it comes from
+		# the fake jid
+		self.groupchat_jids = {} # {ID : groupchat_jid}
 		if USE_GPG:
-			self.gpg = GnuPG.GnuPG()
+			self.gpg = GnuPG.GnuPG(gajim.config.get('use_gpg_agent'))
 			gajim.config.set('usegpg', True)
 		else:
 			gajim.config.set('usegpg', False)
@@ -92,7 +97,7 @@ class Connection(ConnectionHandlers):
 		self.available_transports = {} # list of available transports on this
 		# server {'icq': ['icq.server.com', 'icq2.server.com'], }
 		self.vcard_supported = True
-		self.metacontacts_supported = True
+		self.private_storage_supported = True
 	# END __init__
 
 	def put_event(self, ev):
@@ -107,7 +112,7 @@ class Connection(ConnectionHandlers):
 	def _reconnect(self):
 		# Do not try to reco while we are already trying
 		self.time_to_reconnect = None
-		if self.connected < 2: #connection failed
+		if self.connected < 2: # connection failed
 			gajim.log.debug('reconnect')
 			self.retrycount += 1
 			signed = self.get_signed_msg(self.status)
@@ -145,6 +150,11 @@ class Connection(ConnectionHandlers):
 			if gajim.config.get_per('accounts', self.name, 'autoreconnect'):
 				self.connected = 1
 				self.dispatch('STATUS', 'connecting')
+				if gajim.status_before_autoaway[self.name]:
+					# We were auto away. So go back online
+					self.status = gajim.status_before_autoaway[self.name]
+					gajim.status_before_autoaway[self.name] = ''
+					self.old_show = 'online'
 				# this check has moved from _reconnect method
 				# do exponential backoff until 15 minutes,
 				# then small linear increase
@@ -187,7 +197,7 @@ class Connection(ConnectionHandlers):
 							_('Transport %s answered wrongly to register request: %s')\
 							% (data[0], data[3])))
 						return
-					req = data[1].asDict()
+					req = data[1]
 					req['username'] = self.new_account_info['name']
 					req['password'] = self.new_account_info['password']
 					def _on_register_result(result):
@@ -196,7 +206,7 @@ class Connection(ConnectionHandlers):
 							return
 						self.password = self.new_account_info['password']
 						if USE_GPG:
-							self.gpg = GnuPG.GnuPG()
+							self.gpg = GnuPG.GnuPG(gajim.config.get('use_gpg_agent'))
 							gajim.config.set('usegpg', True)
 						else:
 							gajim.config.set('usegpg', False)
@@ -215,10 +225,7 @@ class Connection(ConnectionHandlers):
 						(data[0], data[3])))
 					return
 				is_form = data[2]
-				if is_form:
-					conf = self.parse_data_form(data[1])
-				else:
-					conf = data[1].asDict()
+				conf = data[1]
 				self.dispatch('REGISTER_AGENT_INFO', (data[0], conf, is_form))
 		elif realm == common.xmpp.NS_PRIVACY:
 			if event == common.xmpp.features_nb.PRIVACY_LISTS_RECEIVED:
@@ -280,16 +287,13 @@ class Connection(ConnectionHandlers):
 	def connect(self, data = None):
 		''' Start a connection to the Jabber server.
 		Returns connection, and connection type ('tls', 'ssl', 'tcp', '')
-		data MUST contain name, hostname, resource, usessl, proxy,
-		use_custom_host, custom_host (if use_custom_host), custom_port (if
-		use_custom_host), '''
+		data MUST contain hostname, usessl, proxy, use_custom_host,
+		custom_host (if use_custom_host), custom_port (if use_custom_host)'''
 		if self.connection:
 			return self.connection, ''
 
 		if data:
-			name = data['name']
 			hostname = data['hostname']
-			resource = data['resource']
 			usessl = data['usessl']
 			self.try_connecting_for_foo_secs = 45
 			p = data['proxy']
@@ -299,9 +303,7 @@ class Connection(ConnectionHandlers):
 				custom_h = data['custom_host']
 				custom_p = data['custom_port']
 		else:
-			name = gajim.config.get_per('accounts', self.name, 'name')
 			hostname = gajim.config.get_per('accounts', self.name, 'hostname')
-			resource = gajim.config.get_per('accounts', self.name, 'resource')
 			usessl = gajim.config.get_per('accounts', self.name, 'usessl')
 			self.try_connecting_for_foo_secs = gajim.config.get_per('accounts',
 				self.name, 'try_connecting_for_foo_secs')
@@ -312,7 +314,7 @@ class Connection(ConnectionHandlers):
 			custom_h = gajim.config.get_per('accounts', self.name, 'custom_host')
 			custom_p = gajim.config.get_per('accounts', self.name, 'custom_port')
 
-		#create connection if it doesn't already exist
+		# create connection if it doesn't already exist
 		self.connected = 1
 		if p and p in gajim.config.get_per('proxies'):
 			proxy = {'host': gajim.config.get_per('proxies', p, 'host')}
@@ -342,8 +344,10 @@ class Connection(ConnectionHandlers):
 		self._hosts = [ {'host': h, 'port': p, 'prio': 10, 'weight': 10} ]
 		self._hostname = hostname
 		if use_srv:
-			# add request for srv query to the resolve, on result '_on_resolve' will be called
-			gajim.resolver.resolve('_xmpp-client._tcp.' + h.encode('utf-8'), self._on_resolve)
+			# add request for srv query to the resolve, on result '_on_resolve'
+			# will be called
+			gajim.resolver.resolve('_xmpp-client._tcp.' + helpers.idn_to_ascii(h),
+				self._on_resolve)
 		else:
 			self._on_resolve('', [])
 
@@ -428,9 +432,8 @@ class Connection(ConnectionHandlers):
 		ConnectionHandlers._register_handlers(self, con, con_type)
 		name = gajim.config.get_per('accounts', self.name, 'name')
 		hostname = gajim.config.get_per('accounts', self.name, 'hostname')
-		resource = gajim.config.get_per('accounts', self.name, 'resource')
 		self.connection = con
-		con.auth(name, self.password, resource, 1, self.__on_auth)
+		con.auth(name, self.password, self.server_resource, 1, self.__on_auth)
 
 	def __on_auth(self, con, auth):
 		if not con:
@@ -458,9 +461,8 @@ class Connection(ConnectionHandlers):
 				self.on_connect_auth(con)
 				self.on_connect_auth = None
 		else:
-			# Forget password if needed
-			if not gajim.config.get_per('accounts', self.name, 'savepass'):
-				self.password = None
+			# Forget password, it's wrong
+			self.password = None
 			gajim.log.debug("Couldn't authenticate to %s" % self._hostname)
 			self.disconnect(on_purpose = True)
 			self.dispatch('STATUS', 'offline')
@@ -488,7 +490,16 @@ class Connection(ConnectionHandlers):
 	def del_privacy_list(self, privacy_list):
 		if not self.connection:
 			return
-		common.xmpp.features_nb.delPrivacyList(self.connection, privacy_list)
+		def _on_del_privacy_list_result(result):
+			if result:
+				self.dispatch('PRIVACY_LIST_REMOVED', privacy_list)
+			else:
+				self.dispatch('ERROR', (_('Error while removing privacy list'),
+					_('Privacy list %s has not been removed. It is maybe active in '
+					'one of your connected resources. Deactivate it and try '
+					'again.') % privacy_list))
+		common.xmpp.features_nb.delPrivacyList(self.connection, privacy_list,
+			_on_del_privacy_list_result)
 	
 	def get_privacy_list(self, title):
 		if not self.connection:
@@ -621,12 +632,6 @@ class Connection(ConnectionHandlers):
 		if not msg:
 			msg = ''
 		keyID = gajim.config.get_per('accounts', self.name, 'keyid')
-		if keyID and USE_GPG and not msg:
-			lowered_uf_status_msg = helpers.get_uf_show(show).lower()
-			# do not show I'm invisible!
-			if lowered_uf_status_msg == _('invisible').lower():
-				lowered_uf_status_msg = _('offline').lower()
-			msg = _("I'm %s") % lowered_uf_status_msg
 		signed = ''
 		if not auto and not show == 'offline':
 			signed = self.get_signed_msg(msg)
@@ -636,6 +641,8 @@ class Connection(ConnectionHandlers):
 			# recconect before we auth to server
 			self.old_show = show
 			self.on_purpose = False 
+			self.server_resource = gajim.config.get_per('accounts', self.name,
+				'resource')
 			self.connect_and_init(show, msg, signed)
 
 		elif show == 'offline' and self.connected:
@@ -696,7 +703,7 @@ class Connection(ConnectionHandlers):
 		self.connection.send(msg_iq)
 
 	def send_message(self, jid, msg, keyID, type = 'chat', subject='',
-	chatstate = None, msg_id = None, composing_jep = None, resource = None,
+	chatstate = None, msg_id = None, composing_xep = None, resource = None,
 	user_nick = None, xhtml = None):
 		if not self.connection:
 			return 1
@@ -721,7 +728,7 @@ class Connection(ConnectionHandlers):
 						' ([This message is *encrypted* (See :JEP:`27`])'
 			else:
 				# Encryption failed, do not send message
-				tim = time.localtime()
+				tim = localtime()
 				self.dispatch('MSGNOTSENT', (jid, error, msgtxt, tim))
 				return 3
 		if msgtxt and not xhtml and gajim.config.get(
@@ -750,12 +757,12 @@ class Connection(ConnectionHandlers):
 		# please note that the only valid tag inside a message containing a <body>
 		# tag is the active event
 		if chatstate is not None:
-			if (composing_jep == 'JEP-0085' or not composing_jep) and \
-			composing_jep != 'asked_once':
-				# JEP-0085
+			if (composing_xep == 'XEP-0085' or not composing_xep) and \
+			composing_xep != 'asked_once':
+				# XEP-0085
 				msg_iq.setTag(chatstate, namespace = common.xmpp.NS_CHATSTATES)
-			if composing_jep in ('JEP-0022', 'asked_once') or not composing_jep:
-				# JEP-0022
+			if composing_xep in ('XEP-0022', 'asked_once') or not composing_xep:
+				# XEP-0022
 				chatstate_node = msg_iq.setTag('x',
 					namespace = common.xmpp.NS_EVENT)
 				if not msgtxt: # when no <body>, add <id>
@@ -779,7 +786,10 @@ class Connection(ConnectionHandlers):
 					kind = 'chat_msg_sent'
 				else:
 					kind = 'single_msg_sent'
-				gajim.logger.write(kind, jid, log_msg)
+				try:
+					gajim.logger.write(kind, jid, log_msg)
+				except exceptions.PysqliteOperationalError, e:
+					self.dispatch('ERROR', (_('Disk Write Error'), str(e)))
 		self.dispatch('MSGSENT', (jid, msg, keyID))
 	
 	def send_stanza(self, stanza):
@@ -877,6 +887,7 @@ class Connection(ConnectionHandlers):
 		if self.connection:
 			return
 		self._hostname = config['hostname']
+		self.server_resource = config['resource']
 		self.new_account_info = config
 		self.name = name
 		self.on_connect_success = self._on_new_account
@@ -897,7 +908,9 @@ class Connection(ConnectionHandlers):
 	def account_changed(self, new_name):
 		self.name = new_name
 
-	def request_last_status_time(self, jid, resource):
+	def request_last_status_time(self, jid, resource, groupchat_jid=None):
+		'''groupchat_jid is used when we want to send a request to a real jid
+		and act as if the answer comes from the groupchat_jid'''
 		if not self.connection:
 			return
 		to_whom_jid = jid
@@ -905,9 +918,15 @@ class Connection(ConnectionHandlers):
 			to_whom_jid += '/' + resource
 		iq = common.xmpp.Iq(to = to_whom_jid, typ = 'get', queryNS =\
 			common.xmpp.NS_LAST)
+		id = self.connection.getAnID()
+		iq.setID(id)
+		if groupchat_jid:
+			self.groupchat_jids[id] = groupchat_jid
 		self.connection.send(iq)
 
-	def request_os_info(self, jid, resource):
+	def request_os_info(self, jid, resource, groupchat_jid=None):
+		'''groupchat_jid is used when we want to send a request to a real jid
+		and act as if the answer comes from the groupchat_jid'''
 		if not self.connection:
 			return
 		# If we are invisible, do not request
@@ -919,6 +938,10 @@ class Connection(ConnectionHandlers):
 			to_whom_jid += '/' + resource
 		iq = common.xmpp.Iq(to = to_whom_jid, typ = 'get', queryNS =\
 			common.xmpp.NS_VERSION)
+		id = self.connection.getAnID()
+		iq.setID(id)
+		if groupchat_jid:
+			self.groupchat_jids[id] = groupchat_jid
 		self.connection.send(iq)
 
 	def get_settings(self):
@@ -1040,12 +1063,15 @@ class Connection(ConnectionHandlers):
 		if password:
 			t.setTagData('password', password)
 		self.connection.send(p)
-		#last date/time in history to avoid duplicate
-		last_log = gajim.logger.get_last_date_that_has_logs(room_jid,
-			is_room = True)
-		if last_log is None:
-			last_log = 0
-		self.last_history_line[room_jid]= last_log
+
+		# last date/time in history to avoid duplicate
+		if not self.last_history_line.has_key(room_jid): 
+			# Not in memory, get it from DB
+			last_log = gajim.logger.get_last_date_that_has_logs(room_jid,
+				is_room = True)
+			if last_log is None:
+				last_log = 0
+			self.last_history_line[room_jid]= last_log
 
 	def send_gc_message(self, jid, msg, xhtml = None):
 		if not self.connection:
@@ -1067,13 +1093,6 @@ class Connection(ConnectionHandlers):
 			to = room_jid)
 		self.connection.send(iq)
 
-	def change_gc_nick(self, room_jid, nick):
-		if not self.connection:
-			return
-		p = common.xmpp.Presence(to = '%s/%s' % (room_jid, nick))
-		p = self.add_sha(p)
-		self.connection.send(p)
-
 	def send_gc_status(self, nick, jid, show, status):
 		if not self.connection:
 			return
@@ -1082,14 +1101,17 @@ class Connection(ConnectionHandlers):
 		ptype = None
 		if show == 'offline':
 			ptype = 'unavailable'
-		show = helpers.get_xmpp_show(show)
+		xmpp_show = helpers.get_xmpp_show(show)
 		p = common.xmpp.Presence(to = '%s/%s' % (jid, nick), typ = ptype,
-			show = show, status = status)
-		if gajim.config.get('send_sha_in_gc_presence'):
+			show = xmpp_show, status = status)
+		if gajim.config.get('send_sha_in_gc_presence') and show != 'offline':
 			p = self.add_sha(p, ptype != 'unavailable')
 		# send instantly so when we go offline, status is sent to gc before we
 		# disconnect from jabber server
 		self.connection.send(p)
+		# Save the time we quit to avoid duplicate logs AND be faster than 
+		# get that date from DB
+		self.last_history_line[jid] = time_time()
 
 	def gc_set_role(self, room_jid, nick, role, reason = ''):
 		'''role is for all the life of the room so it's based on nick'''
@@ -1139,11 +1161,12 @@ class Connection(ConnectionHandlers):
 		item.setAttr('affiliation', affiliation)
 		self.connection.send(iq)
 
-	def send_gc_config(self, room_jid, config):
+	def send_gc_config(self, room_jid, form):
 		iq = common.xmpp.Iq(typ = 'set', to = room_jid, queryNS =\
 			common.xmpp.NS_MUC_OWNER)
 		query = iq.getTag('query')
-		self.build_data_from_dict(query, config)
+		form.setAttr('type', 'submit')
+		query.addChild(node = form)
 		self.connection.send(iq)
 
 	def gpg_passphrase(self, passphrase):
