@@ -31,6 +31,7 @@ except ImportError:
 
 import configpaths
 LOG_DB_PATH = configpaths.gajimpaths['LOG_DB']
+LOG_DB_FOLDER, LOG_DB_FILE = os.path.split(LOG_DB_PATH)
 
 class Constants:
 	def __init__(self):
@@ -97,10 +98,18 @@ class Logger:
 	def open_db(self):
 		self.close_db()
 
+		# FIXME: sqlite3_open wants UTF8 strings. So a path with
+		# non-ascii chars doesn't work. See #2812 and
+		# http://lists.initd.org/pipermail/pysqlite/2005-August/000134.html
+		back = os.getcwd()
+		os.chdir(LOG_DB_FOLDER)
+
 		# if locked, wait up to 20 sec to unlock
 		# before raise (hopefully should be enough)
-		self.con = sqlite.connect(LOG_DB_PATH, timeout = 20.0,
+
+		self.con = sqlite.connect(LOG_DB_FILE, timeout = 20.0,
 			isolation_level = 'IMMEDIATE')
+		os.chdir(back)
 		self.cur = self.con.cursor()
 		self.set_synchronous(False)
 
@@ -160,19 +169,27 @@ class Logger:
 				jid = jid.split('/', 1)[0] # remove the resource
 		if jid in self.jids_already_in: # we already have jids in DB
 			self.cur.execute('SELECT jid_id FROM jids WHERE jid=?', [jid])
-			jid_id = self.cur.fetchone()[0]
-		else: # oh! a new jid :), we add it now
-			if typestr == 'ROOM':
-				typ = constants.JID_ROOM_TYPE
-			else:
-				typ = constants.JID_NORMAL_TYPE
-			self.cur.execute('INSERT INTO jids (jid, type) VALUES (?, ?)', (jid, typ))
-			try:
-				self.con.commit()
-			except sqlite.OperationalError, e:
-				print >> sys.stderr, str(e)
-			jid_id = self.cur.lastrowid
-			self.jids_already_in.append(jid)
+			row = self.cur.fetchone()
+			if row:
+				return row[0]
+		# oh! a new jid :), we add it now
+		if typestr == 'ROOM':
+			typ = constants.JID_ROOM_TYPE
+		else:
+			typ = constants.JID_NORMAL_TYPE
+		try:
+			self.cur.execute('INSERT INTO jids (jid, type) VALUES (?, ?)', (jid,
+				typ))
+		except sqlite.IntegrityError, e:
+			# Jid already in DB, maybe added by another instance. re-read DB
+			self.get_jids_already_in_db()
+			return self.get_jid_id(jid, typestr)
+		try:
+			self.con.commit()
+		except sqlite.OperationalError, e:
+			print >> sys.stderr, str(e)
+		jid_id = self.cur.lastrowid
+		self.jids_already_in.append(jid)
 		return jid_id
 	
 	def convert_human_values_to_db_api_values(self, kind, show):
@@ -276,7 +293,10 @@ class Logger:
 	def commit_to_db(self, values, write_unread = False):
 		#print 'saving', values
 		sql = 'INSERT INTO logs (jid_id, contact_name, time, kind, show, message, subject) VALUES (?, ?, ?, ?, ?, ?, ?)'
-		self.cur.execute(sql, values)
+		try:
+			self.cur.execute(sql, values)
+		except sqlite.OperationalError, e:
+			raise exceptions.PysqliteOperationalError(str(e))
 		message_id = None
 		try:
 			self.con.commit()
@@ -416,10 +436,11 @@ class Logger:
 		# 3 - 8 (we avoid the last 2 lines but we still return 5 asked)
 		self.cur.execute('''
 			SELECT time, kind, message FROM logs
-			WHERE (%s) AND kind IN (%d, %d, %d, %d) AND time > %d
+			WHERE (%s) AND kind IN (%d, %d, %d, %d, %d) AND time > %d
 			ORDER BY time DESC LIMIT %d OFFSET %d
-			''' % (where_sql, constants.KIND_SINGLE_MSG_RECV, constants.KIND_CHAT_MSG_RECV,
-				constants.KIND_SINGLE_MSG_SENT, constants.KIND_CHAT_MSG_SENT,
+			''' % (where_sql, constants.KIND_SINGLE_MSG_RECV,
+				constants.KIND_CHAT_MSG_RECV, constants.KIND_SINGLE_MSG_SENT,
+				constants.KIND_CHAT_MSG_SENT, constants.KIND_ERROR,
 				timed_out, restore_how_many_rows, pending_how_many)
 			)
 
@@ -474,7 +495,7 @@ class Logger:
 			
 		else: # user just typed something, we search in message column
 			where_sql = self._build_contact_where(account, jid)
-			like_sql = '%' + query + '%'
+			like_sql = '%' + query.replace("'", "''") + '%'
 			self.cur.execute('''
 				SELECT contact_name, time, kind, show, message, subject FROM logs
 				WHERE (%s) AND message LIKE '%s'
@@ -506,7 +527,7 @@ class Logger:
 			constants.KIND_STATUS, constants.KIND_GCSTATUS))
 		result = self.cur.fetchall()
 
-		# Copy all interesant time in a temporary table 
+		# Copy all interesting times in a temporary table 
 		self.cur.execute('CREATE TEMPORARY TABLE blabla(time,INTEGER)') 
 		for line in result: 
 			self.cur.execute(''' 
@@ -545,10 +566,9 @@ class Logger:
 			jid_id = self.get_jid_id(jid, 'ROOM')
 			where_sql = 'jid_id = %s' % jid_id	
 		self.cur.execute('''
-			SELECT time FROM logs
+			SELECT MAX(time) FROM logs
 			WHERE (%s) 
 			AND kind NOT IN (%d, %d)
-			ORDER BY time DESC LIMIT 1
 			''' % (where_sql, constants.KIND_STATUS, constants.KIND_GCSTATUS))
 
 		results = self.cur.fetchone()
