@@ -63,6 +63,8 @@ class BaseTooltip:
 		self.win.set_border_width(3)
 		self.win.set_resizable(False)
 		self.win.set_name('gtk-tooltips')
+		if gtk.gtk_version >= (2, 10, 0) and gtk.pygtk_version >= (2, 10, 0):
+			self.win.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_TOOLTIP)
 		
 		self.win.set_events(gtk.gdk.POINTER_MOTION_MASK)
 		self.win.connect_after('expose_event', self.expose)
@@ -298,7 +300,7 @@ class GCTooltip(BaseTooltip):
 				# escape markup entities
 				status = helpers.reduce_chars_newlines(status, 100, 5)
 				status = '<i>' +\
-					 gtkgui_helpers.escape_for_pango_markup(status) + '</i>'
+					gobject.markup_escape_text(status) + '</i>'
 				properties.append((status, None))
 		else: # no status message, show SHOW instead
 			show = helpers.get_uf_show(contact.show)
@@ -588,7 +590,7 @@ class FileTransfersTooltip(BaseTooltip):
 			status =  _('Not started')
 		elif file_props.has_key('connected'):
 			if file_props.has_key('stopped') and \
-				file_props['stopped'] == True:
+			file_props['stopped'] == True:
 				status = _('Stopped')
 			elif file_props['completed']:
 					status = _('Completed')
@@ -597,10 +599,10 @@ class FileTransfersTooltip(BaseTooltip):
 					status = _('Completed')
 			else:
 				if file_props.has_key('paused') and  \
-					file_props['paused'] == True:
-                                        status = _('?transfer status:Paused')
+				file_props['paused'] == True:
+					status = _('?transfer status:Paused')
 				elif file_props.has_key('stalled') and \
-					file_props['stalled'] == True:
+				file_props['stalled'] == True:
 					#stalled is not paused. it is like 'frozen' it stopped alone
 					status = _('Stalled')
 				else:
@@ -615,13 +617,13 @@ class FileTransfersTooltip(BaseTooltip):
 			label.set_alignment(0, 0)
 			label.set_markup(property[0])
 			ft_table.attach(label, 1, 2, current_row, current_row + 1, 
-																gtk.FILL,  gtk.FILL, 0, 0)
+				gtk.FILL,  gtk.FILL, 0, 0)
 			label = gtk.Label()
 			label.set_alignment(0, 0)
 			label.set_line_wrap(True)
 			label.set_markup(property[1])
 			ft_table.attach(label, 2, 3, current_row, current_row + 1, 
-											gtk.EXPAND | gtk.FILL, gtk.FILL, 0, 0)
+				gtk.EXPAND | gtk.FILL, gtk.FILL, 0, 0)
 		
 		self.win.add(ft_table)
 
